""" A multi function mapping UFL expressions to pymbolic expressions

This mapper only defines handler for those UFL expression types that have an
equivalent in pymbolic (mainly algebraic operations).

This is mainly intended as a base class for anything mapping ufl to pymbolic.
"""

from __future__ import absolute_import
from ufl.algorithms import MultiFunction
from pymbolic.primitives import Product, Quotient, Subscript, Sum, Variable, Call

# The constructed pymbolic expressions use n-ary operators instead of ufls binary operators
from dune.perftool.ufl.flatoperators import get_operands


class UFL2PymbolicMapper(MultiFunction):
    def __init__(self):
        super(UFL2PymbolicMapper, self).__init__()

    call = MultiFunction.__call__

    def product(self, o):
        return Product(tuple(self.call(op) for op in get_operands(o)))

    def float_value(self, o):
        return o.value()

    def int_value(self, o):
        return o.value()

    def division(self, o):
        assert len(o.ufl_operands) == 2

        return Quotient(self.call(o.ufl_operands[0]), self.call(o.ufl_operands[1]))

    def sum(self, o):
        return Sum(tuple(self.call(op) for op in get_operands(o)))

    def zero(self, o):
        return 0

    def abs(self, o):
        from ufl.classes import JacobianDeterminant
        if isinstance(o.ufl_operands[0], JacobianDeterminant):
            return self.call(o.ufl_operands[0])
        else:
            return Call('abs', self.call(o.ufl_operands[0]))
